// イメージエディタ 8x8版 & カラーピッカー
// 公開された8x8ドット イメージデータを一覧・編集・拡大表示します
// ※別途イメージデータファイル(Img8x8.int)が必要です
// STARTボタンを押すと終了します

// イメージデータについて
// http://www.picosoft.co.jp/ORANGE-light/SystemCharacters.pdf
// Copyright (c) 2019 Picosoft Co., Ltd.
// Made by Kaori Hanatsumi

//下記"18"を、IMG8x8c.intを保存したEEPROMスロット番号に書き換えてください
FSLOT = 18-1

' 色定数の定義
WHITE = rgb(255, 255, 255)
DARKGRAY = rgb(64, 64, 64)
ORANGE = rgb(255, 163, 0)
LBLUE = rgb(0, 140, 255)
DARKBLUE = rgb(0, 0, 64)
RED = rgb(255, 0, 0)

' ボタンの定義
BUTTON_UP = 5
BUTTON_DOWN = 8
BUTTON_LEFT = 7
BUTTON_RIGHT = 6
BUTTON_A = 11
BUTTON_B = 10
BUTTON_START = 9

' スプライトパターン定義
IMG_CSR = getmptr()
mdata 0xFE,0x7C,0x38,0x10

' ワークエリア定義
OFFSET = getmptr()
mptr OFFSET + 2
SBUF = getmptr()
mptr SBUF + 2
OFFSET2 = getmptr()
mptr OFFSET2 + 2
BUF = getmptr()
mptr BUF + 8

i2ci 1,400

' 初期画面 描画
cls 2
for i=0 to 199
	img_get(i,0)
next

gprint 0,232,"Push START to Exit",ORANGE,0

' スプライト作成・表示
csr_r = sprnew(0,90-3,24*8-4,8,4,IMG_CSR,WHITE,0)
csr_g = sprnew(1,90-3,26*8-4,8,4,IMG_CSR,WHITE,0)
csr_b = sprnew(2,90-3,28*8-4,8,4,IMG_CSR,WHITE,0)
csr_sel = sprnew(3,90-3,24*8-4,8,4,IMG_CSR,RED,0)
sprdsp csr_r,1
sprdsp csr_g,1
sprdsp csr_b,1

' カラーバー描画
for i=0 to 63
	j=90+i
	rect j,24*8,1,7,rgb(i*4,0,0),1
	rect j,26*8,1,7,rgb(0,i*4,0),1
	rect j,28*8,1,7,rgb(0,0,i*4),1
next

' メインループ
i=0:img_zoom(i)
mode = 0
sel=0
while in(BUTTON_START)
	if trig(BUTTON_B) then change_mode()
	if mode = 0 then sel_img()
	if mode = 1 then edit_img()
	if mode = 2 then edit_color():pause 20
wend
end

' 動作モードを移行する
def change_mode() {
	mode = (mode + 1) % 3
// 編集開始
	if mode=1 then csr_view(x,y,RED) : return
// 編集完了、色編集開始
	if mode=2 then csr_view(x,y,0) : img_write(i) : csr_on() : return
// 色編集完了 : 色確定
	csr_off()
	pre_col = rgb(r*4, g*4, b*4)
	mpokew SBUF/2 ,pre_col
	index_write(i)
	img_zoom(i)
}

' イメージを選択する
def sel_img() {
	var pre_i
	pre_i = i
	i = i + trig(BUTTON_RIGHT) - trig(BUTTON_LEFT) + trig(BUTTON_DOWN)*10 - trig(BUTTON_UP)*10
	i = i + ((i<0) - (i>199))*200
	if i <> pre_i then img_get(pre_i,0):img_zoom(i)
	if trig(BUTTON_A) then mpokew SBUF/2 ,pre_col:index_write(i):img_zoom(i)
}

' イメージデータを読んで表示
def img_get(n,flag) {
	var col1
	var col2
	eeprom_read(FSLOT*2048 + 1632 + n*2,SBUF, 2)
	eeprom_read(FSLOT*2048 + 32 + n*8, BUF, 8)
	col = mpeekw(SBUF/2)
	col1 = col : col2 = DARKBLUE
	if flag then col1=DARKGRAY : col2=ORANGE
	sprnew(4,(10 + n%10)*16,(n/10)*12,8,8,BUF,col1,col2)
	sprdsp 4,1
	return 0
}

' イメージデータを読んで拡大表示
def img_zoom(n) {
	var cx
	var cy
	var d
	if img_get(n,1) then rect 0,11,80,80,0,1 : return
	set_color_all(col)
	cy=7
	while cy>=0
		d=mpeek(BUF + cy)
		cx=7
		while cx>=0
			rect cx*10+1, cy*10+11, 8, 8, col*(d&1), 1
			d=d>>1:cx=cx-1
		wend
		cy=cy-1
	wend
}

' EEPROMからptrにlenバイト分 読み込む
def eeprom_read(adr,ptr,len) {
	var e
	mpokew OFFSET/2,adr
	e = i2cw(1, 0x50, OFFSET, 2, 0)
	e = i2cr(1, 0x50, ptr, len, 1)
}

' 色コードをEEPROMに書き込む
def index_write(id) {
	var e
	mpokew OFFSET/2,FSLOT*2048 + 1632 + id*2
	e = i2cw(1,0x50,OFFSET,4,1)
}

' イメージをEEPROMに保存する
def img_write(id) {
	var e
	mpokew OFFSET2/2, FSLOT*2048 + 32 + id*8
	e = i2cw(1, 0x50, OFFSET2, 10, 1)
}

' イメージを編集する
def edit_img() {
	move_csr()
	if trig(BUTTON_A) then pbit = !bit_get(x,y)
	if !in(BUTTON_A) then bit_set(x,y,pbit)
}

' イメージ編集用カーソルを移動する
def move_csr() {
	var bx
	var by
	bx = x
	by = y
	x = x - trig(BUTTON_LEFT) + trig(BUTTON_RIGHT)
	y = y - trig(BUTTON_UP) + trig(BUTTON_DOWN)
	x = x + ((x<0) - (x>7)) * 8
	y = y + ((y<0) - (y>7)) * 8
	if (bx != x) || (by != y) then csr_view(bx,by,0):csr_view(x,y,RED)
}

' イメージ編集用カーソルを描画する
def csr_view(x,y,c) {
	rect x*10, y*10+10, 10, 10, c, 0
}

' イメージバッファからx,yのドットを取得
def bit_get(x,y) {
	return (mpeek(BUF + y) >> (7-x)) & 1
}
' イメージバッファのx,yにドットを設定、拡大画面にも描画する
def bit_set(x,y,b) {
	var adr
	var n
	n= 7 - x
	adr = BUF + y
	mpoke adr,mpeek(adr) & ~(1<<n) | (b<<n)
	rect x*10+1, y*10+11 ,8 ,8 ,col*b ,1
}

' 色を編集する
def edit_color() {
	change_sel(trig(BUTTON_DOWN) - trig(BUTTON_UP))
	change_col(!in(BUTTON_RIGHT)*(s<63) - !in(BUTTON_LEFT)*(s>0))
}

' 色の変更
def change_col(d) {
	if d=0 then return
	s=s+d
	sprmov csr_sel,87+s,24*8-4 + sel*16
	set_color(s)
	rect 65,24*8,20,40,rgb(r*4,g*4,b*4),1
}

' RGBバー間のカーソル移動
def change_sel(d) {
	if d=0 then return
	csr_off()
	sel=(sel+d+3)%3
	csr_on()
}

' 色選択カーソルを非表示にする
def csr_off() {
	sprdsp csr_sel,0
	sprmov sel,87+s,24*8-4 + sel*16
	sprdsp sel,1
}

' 色選択カーソルを表示する
def csr_on() {
	s=get_color()
	sprdsp sel,0
	sprmov csr_sel,87+s,24*8-4 + sel*16
	sprdsp csr_sel,1
}

' 選択している色の輝度を取得
def get_color() {
	if sel=0 then return r
	if sel=1 then return g
	if sel=2 then return b
}

' 選択している色の輝度を変更
def set_color(n) {
	if sel=0 then r=n
	if sel=1 then g=n
	if sel=2 then b=n
}

' 色コードからrgbをまとめて設定
def set_color_all(c) {
	r = (c >> 10) & 0x3e
	g = (c >> 5) & 0x3f
	b = (c << 1) & 0x3e
	sprmov csr_r,87+r,24*8-4
	sprmov csr_g,87+g,26*8-4
	sprmov csr_b,87+b,28*8-4
	rect 65,24*8,20,40,c,1
}

' ボタン状態が変化＆押された時だけ1になる関数
def trig(n) {
	var ret
	ret = pre
	pre = pre & ~(1 << n) | (in(n) << n)
	return ((ret & ~pre) >> n) & 1
}
